@@ -611,11 +611,18 @@ bool AlsaSoundDevice::noticeSoundStreamData(SoundStreamID id,
 }
 
 
+// pure virtual members of ThreadLoggingClient
+IErrorLogClient *AlsaSoundDevice::getErrorLogClient()
+{
+    return this;
+}
 
 void AlsaSoundDevice::slotPollPlayback()
 {
     //     logDebug("AlsaSoundDevice::slotPollPlayback()");
     BlockProfiler p("AlsaSoundDevice::slotPollPlayback()");
+    
+//     printf ("slotPollPlayback\n");      
 
     if (m_PlaybackStreamID.isValid()) {
 
@@ -653,7 +660,9 @@ void AlsaSoundDevice::slotPollPlayback()
         }
         checkThreadErrorsAndWarning();
 
+//         printf ("    slotPollPlayback: before buffer size check: free size = %zi, size = %zi, free ratio = %4.1f %%\n", m_PlaybackBuffer.getFreeSize(), m_PlaybackBuffer.getSize(), (double)m_PlaybackBuffer.getFreeSize() / (double) m_PlaybackBuffer.getSize() * 100.0 );      
         if (m_PlaybackBuffer.getFreeSize() > m_PlaybackBuffer.getSize() / 3) {
+//             printf ("    slotPollPlayback: buffer check successful: requesting data\n");      
             notifyReadyForPlaybackData(m_PlaybackStreamID, m_PlaybackBuffer.getFreeSize());
         }
 
@@ -775,6 +784,7 @@ bool AlsaSoundDevice::openPlaybackDevice(const SoundFormat &format, bool reopen)
                 m_playbackThread = NULL;
             }
             m_playbackThread = new AlsaThread(this, /*playback_not_capture = */ true, m_hPlayback, m_PlaybackFormat);
+            m_playbackThread->setLatency(qMin(50u, m_PlaybackLatency) * 1000);
             m_playbackThread->start();
             m_PlaybackPollingTimer.start(50); // polling still necessary, however mainly for pushing sound around and getting volume
         } else {
@@ -837,6 +847,7 @@ bool AlsaSoundDevice::openCaptureDevice(const SoundFormat &format, bool reopen)
                 m_captureThread = NULL;
             }
             m_captureThread = new AlsaThread(this, /*playback_not_capture = */ false, m_hCapture, m_CaptureFormat);
+            m_captureThread->setLatency(qMin(50u, m_CaptureLatency) * 1000);
             m_captureThread->start();
             m_CapturePollingTimer.start(50); // polling still necessary, however mainly for pushing sound around and getting volume
         } else {
@@ -973,7 +984,9 @@ bool AlsaSoundDevice::openAlsaDevice(snd_pcm_t *&alsa_handle, SoundFormat &forma
 
     if (!error) {
         snd_pcm_prepare(alsa_handle);
+#ifdef DEBUG
         snd_pcm_dump(alsa_handle, log);
+#endif
     }
 
     return !error;
@@ -1870,24 +1883,12 @@ void   AlsaSoundDevice::unlockCaptureBufferTransaction()
 void  AlsaSoundDevice::checkThreadErrorsAndWarning()
 {
     if (m_captureThread) {
-        if (m_captureThread->error()) {
-            logError  (i18n("AlsaPlugin(capture thread on %1): %2", m_CaptureDeviceName, m_captureThread->errorString()));
-            m_captureThread->resetError();
-        }
-        if (m_captureThread->warning()) {
-            logWarning(i18n("AlsaPlugin(capture thread on %1): %2", m_CaptureDeviceName, m_captureThread->warningString()));
-            m_captureThread->resetWarning();
-        }
+        checkLogs(m_captureThread, i18n("AlsaPlugin(capture thread on %1): ",   m_CaptureDeviceName),    /*resetLogs = */ true);
+        m_captureThread->resetError();
     }
     if (m_playbackThread) {
-        if (m_playbackThread->error()) {
-            logError  (i18n("AlsaPlugin(playback thread on %1): %2", m_CaptureDeviceName, m_playbackThread->errorString()));
-            m_playbackThread->resetError();
-        }
-        if (m_playbackThread->warning()) {
-            logWarning(i18n("AlsaPlugin(playback thread on %1): %2", m_CaptureDeviceName, m_playbackThread->warningString()));
-            m_playbackThread->resetWarning();
-        }
+        checkLogs(m_playbackThread, i18n("AlsaPlugin(playback thread on %1): ", m_PlaybackDeviceName), /*resetLogs = */ true);
+        m_playbackThread->resetError();
     }
 }
 
