@@ -20,34 +20,31 @@
 #endif
 
 #include <QtCore/QList>
-#include <QtCore/QXmlStreamReader>
-#include <QtCore/QTextCodec>
 
 #include <kconfiggroup.h>
 #include <kiconloader.h>
 #include <kaboutdata.h>
 #include <klocale.h>
-#include <kio/jobclasses.h>
-#include <kio/job.h>
-#include <ktemporaryfile.h>
-#include <kencodingprober.h>
 
 #include <unistd.h>
 
 #include "stationlist.h"
 #include "utils.h"
 #include "internetradio.h"
-#include "soundstream_decoding_step_event.h"
 #include "decoder_thread.h"
 
 #include "internetradio-configuration.h"
+#include "icy_http_handler.h"
+#include "mmsx_handler.h"
 
 
 #ifdef KRADIO_ENABLE_FIXMES
     #warning "FIXME: make buffer size configurable"
 #endif
-#define MAX_BUFFERS           20
-#define MIN_BUFFERS4PLAYBACK  10
+#define MAX_BUFFER_SIZE       (256*1024)
+#define MAX_BUFFERS           8
+#define MAX_BYTES_PER_BUFFER  ((MAX_BUFFER_SIZE) / (MAX_BUFFERS))
+#define MIN_BUFFERS4PLAYBACK  2
 
 ///////////////////////////////////////////////////////////////////////
 
@@ -63,10 +60,7 @@ InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
     m_decoderThread(NULL),
     m_currentStation(InternetRadioStation()),
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    m_currentStreamIdx(-1),
-    m_currentStreamRetriesMax(2),
-    m_currentStreamRetriesLeft(-1),
-    m_randStreamIdxOffset(-1),
+    m_streamReader             (NULL),
 #endif
     m_stereoFlag(false),
     m_muted(false),
@@ -78,23 +72,27 @@ InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
     m_RDS_visible(false),
     m_RDS_StationName(QString()),
     m_RDS_RadioText(QString()),
-    m_maxStreamProbeSize(4096),
-    m_maxStreamAnalyzeTime(0.5),
-    m_waitForBufferMinFill(true)/*,
-    m_MimetypeJob(NULL)*/,
-    m_playlistJob(NULL)
-#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    , m_streamJob(NULL)
-#endif
+    m_maxStreamProbeSize(8192),
+    m_maxStreamAnalyzeTime(0.8),
+    m_waitForBufferMinFill(true)
 {
     m_SoundStreamSinkID   = createNewSoundStream(false);
     m_SoundStreamSourceID = m_SoundStreamSinkID;
+
+    QObject::connect(&m_playlistHandler, SIGNAL(sigEOL()),                      this, SLOT(slotPlaylistEOL()));
+    QObject::connect(&m_playlistHandler, SIGNAL(sigError(QString)),             this, SLOT(slotPlaylistError(QString)));
+    QObject::connect(&m_playlistHandler, SIGNAL(sigPlaylistLoaded(KUrl::List)), this, SLOT(slotPlaylistLoaded(KUrl::List)));
+    QObject::connect(&m_playlistHandler, SIGNAL(sigStreamSelected(KUrl)),       this, SLOT(slotPlaylistStreamSelected(KUrl)));
 }
 
 
 InternetRadio::~InternetRadio()
 {
     setPower(false);
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    delete m_streamReader;
+    m_streamReader    = NULL;
+#endif
 }
 
 
@@ -277,11 +275,6 @@ bool InternetRadio::powerOff ()
     if (! isPowerOn())
         return true;
 
-    loadPlaylistStopJob();
-#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    stopStreamDownload();
-#endif
-
     queryPlaybackVolume(m_SoundStreamSinkID, m_defaultPlaybackVolume);
     if (m_PlaybackMixerMuteOnPowerOff) {
       sendMuteSink(m_SoundStreamSourceID);
@@ -476,6 +469,15 @@ bool    InternetRadio::isSourceMuted(SoundStreamID id, bool &m) const
 }
 
 
+
+
+// pure virtual members of ThreadLoggingClient
+IErrorLogClient    *InternetRadio::getErrorLogClient()
+{
+    return this;
+}
+
+
 // IInternetRadioCfg methods
 
 static inline void assignChannelIfValid(QString &dest_channel, const QString &test_channel, const QStringList &valid_channels)
@@ -486,7 +488,7 @@ static inline void assignChannelIfValid(QString &dest_channel, const QString &te
 }
 
 
-bool  InternetRadio::setPlaybackMixer(const QString &soundStreamClientID, const QString &ch, bool muteOnPowerOff, bool force)
+bool  InternetRadio::setPlaybackMixer(QString soundStreamClientID, QString ch, bool muteOnPowerOff, bool force)
 {
     QString old_channel           = m_PlaybackMixerChannel;
     m_PlaybackMixerID             = soundStreamClientID;
@@ -552,10 +554,10 @@ void   InternetRadio::saveState (KConfigGroup &config) const
     config.writeEntry("defaultPlaybackVolume",       m_defaultPlaybackVolume);
     config.writeEntry("URL",                         m_currentStation.url());
     config.writeEntry("PowerOn",                     isPowerOn());
-    config.writeEntry("maxStreamProbeSize",          m_maxStreamProbeSize);
-    config.writeEntry("maxStreamAnalyzeTime",        m_maxStreamAnalyzeTime);
+    config.writeEntry("maxStreamProbeSizeNew",       m_maxStreamProbeSize);
+    config.writeEntry("maxStreamAnalyzeTimeNew",     m_maxStreamAnalyzeTime);
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    config.writeEntry("maxStreamRetries",            m_currentStreamRetriesMax);
+    config.writeEntry("maxStreamRetries",            m_maxStreamRetries);
 #endif
 
     saveRadioDeviceID(config);
@@ -573,10 +575,10 @@ void   InternetRadio::restoreState (const KConfigGroup &config)
     m_defaultPlaybackVolume       = config.readEntry ("defaultPlaybackVolume",       0.5);
 
 
-    m_maxStreamProbeSize          = config.readEntry ("maxStreamProbeSize",          4096);
-    m_maxStreamAnalyzeTime        = config.readEntry ("maxStreamAnalyzeTime",        0.5);
+    m_maxStreamProbeSize          = config.readEntry ("maxStreamProbeSizeNew",       8192);
+    m_maxStreamAnalyzeTime        = config.readEntry ("maxStreamAnalyzeTimeNew",     0.8);
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    m_currentStreamRetriesMax     = config.readEntry ("maxStreamRetries",            2);
+    m_maxStreamRetries            = config.readEntry ("maxStreamRetries",            2);
 #endif
 
 
@@ -640,36 +642,123 @@ void InternetRadio::radio_init()
     m_stereoFlag = false;
     freeAllBuffers(); // just to be sure;-)
 
-//     m_MimetypeJob = KIO::mimetype(m_currentStation.url());
-//     QObject::connect(m_MimetypeJob, SIGNAL(mimetype(KIO::Job *, const QString &)), this, SLOT(slotMimetypeResult(KIO::Job *, const QString &)));
-//     m_MimetypeJob->start();
-
-//     m_decoderThread = new DecoderThread(this, m_currentStation, MAX_BUFFERS, m_maxStreamProbeSize, m_maxStreamAnalyzeTime);
-//     m_decoderThread->start();
-
     m_waitForBufferMinFill = true;
     m_powerOn              = true;
 
-    loadPlaylistStartJob();
+    m_playlistHandler.setPlayListUrl(m_currentStation, m_maxStreamRetries);
+    m_playlistHandler.startPlaylistDownload();
+
     logDebug(QString("InternetRadio::radio_init"));
 }
 
 
-void InternetRadio::startDecoderThread()
+
+void InternetRadio::slotPlaylistLoaded(KUrl::List playlist)
+{
+    m_currentPlaylist = playlist;
+#ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    startDecoderThread();
+#else
+    m_playlistHandler.selectNextStream(true, true);
+#endif
+}
+
+
+void InternetRadio::slotPlaylistStreamSelected(KUrl stream)
 {
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
-    startStreamDownload();
+    stopStreamReader();
+    stopDecoderThread();
+    startStreamReader(stream);
 #endif
+}
+
+
+void InternetRadio::slotPlaylistError(QString /*errorMsg*/)
+{
+    // no extra error msg, messages have already been issued by the playlist handler
+    powerOff();
+}
+
+
+void InternetRadio::slotPlaylistEOL()
+{
+    powerOff();
+}
+
+
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+void InternetRadio::startStreamReader(KUrl stream)
+{
+    stopStreamReader();
+    if (stream.protocol() == "mms" || stream.protocol() == "mmsx") {
+        m_streamReader = new MMSXHandler();
+    } else {
+        m_streamReader = new IcyHttpHandler();
+    }
+    connect(m_streamReader, SIGNAL(sigMetaDataUpdate(KIO::MetaData)),             this, SLOT(slotMetaDataUpdate(KIO::MetaData)));
+    connect(m_streamReader, SIGNAL(sigError(KUrl)),                               this, SLOT(slotStreamError(KUrl)));
+    connect(m_streamReader, SIGNAL(sigFinished(KUrl)),                            this, SLOT(slotStreamFinished(KUrl)));
+    connect(m_streamReader, SIGNAL(sigStarted(KUrl)),                             this, SLOT(slotStreamStarted(KUrl)));
+    connect(m_streamReader, SIGNAL(sigUrlChanged(KUrl)),                          this, SLOT(slotInputStreamUrlChanged(KUrl)));
+    connect(m_streamReader, SIGNAL(sigConnectionEstablished(KUrl,KIO::MetaData)), this, SLOT(slotStreamConnectionEstablished(KUrl,KIO::MetaData)));
+
+    m_streamReader->startStreamDownload(stream);
+}
+
+
+void InternetRadio::stopStreamReader()
+{
+    if (m_streamReader) {
+        m_streamReader->stopStreamDownload();
+        m_streamReader->disconnect(this);
+        m_streamReader->deleteLater();
+        m_streamReader = NULL;
+    }
+}
+
+#endif
+
+
+void InternetRadio::startDecoderThread()
+{
     if (m_decoderThread) {
         m_decoderThread->quit();
     }
-    m_decoderThread = new DecoderThread(this, m_currentStation, m_currentPlaylist, MAX_BUFFERS, m_maxStreamProbeSize, m_maxStreamAnalyzeTime);
+    m_decoderThread = new DecoderThread(this,
+                                        m_currentStation,
+#ifdef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+                                        m_currentPlaylist,
+#else
+                                        m_playlistHandler.currentStreamUrl(),
+                                        m_streamReader,
+//                                         m_streamInputBuffer,
+#endif
+                                        MAX_BUFFERS,
+                                        MAX_BYTES_PER_BUFFER,
+                                        m_maxStreamProbeSize,
+                                        m_maxStreamAnalyzeTime
+                                       );
     QObject::connect(m_decoderThread, SIGNAL(finished()),   this, SLOT(slotDecoderThreadFinished()));
     QObject::connect(m_decoderThread, SIGNAL(terminated()), this, SLOT(slotDecoderThreadFinished()));
     m_decoderThread->start();
 }
 
 
+void InternetRadio::stopDecoderThread()
+{
+    checkDecoderMessages();
+    if (m_decoderThread && m_decoderThread->decoder()) {
+        m_decoderThread->decoder()->setDone();
+    }
+    if (m_decoderThread) {
+        m_decoderThread->quit();
+        // delete m_decoderThread; // the thread will delete itself, but only when it really finished.
+        m_decoderThread = NULL;
+    }
+}
+
+
 void InternetRadio::slotDecoderThreadFinished()
 {
     // to avoid orphaned thread objects, we will delete finished threads here.
@@ -681,6 +770,9 @@ void InternetRadio::slotDecoderThreadFinished()
     DecoderThread *t = static_cast<DecoderThread*>(sender());
     if (t == m_decoderThread) {
         m_decoderThread = NULL;
+        if (isPowerOn()) {
+            powerOff();
+        }
     }
     t->deleteLater();
 }
@@ -691,19 +783,12 @@ void InternetRadio::radio_done()
     freeAllBuffers();
     m_powerOn    = false;
     m_stereoFlag = false;
-    if (m_decoderThread && m_decoderThread->decoder()) {
-        m_decoderThread->decoder()->setDone();
-    }
-    if (m_decoderThread) {
-        m_decoderThread->quit();
-/*        if (m_decoderThread->isRunning() && !m_decoderThread->wait(500)) { // wait at max 500 ms, otherwise we'll kill you ;-)
-            m_decoderThread->terminate();
-            m_decoderThread->wait(1000);
-            logWarning(i18n("I'm sorry, but I'd to kill the ffmpeg decoder thread. It might be a good idea to restart KRadio"));
-        }*/
-        // delete m_decoderThread; // the thread will delete itself, but only when it really finished.
-        m_decoderThread = NULL;
-    }
+
+    m_playlistHandler.stopPlaylistDownload();
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    stopStreamReader();
+#endif
+    stopDecoderThread();
 }
 
 
@@ -718,6 +803,7 @@ bool InternetRadio::setPlaybackVolume(SoundStreamID id, float volume)
     }
 }
 
+
 bool InternetRadio::getPlaybackVolume(SoundStreamID id, float &volume) const
 {
     if (isPowerOff() && id == m_SoundStreamSinkID) {
@@ -824,8 +910,9 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
     while (m_decoderThread && (n_bufs = m_decoderThread->decoder()->availableBuffers()) && (free_size > 0) && (consumed_size != 0)) {
 
         DataBuffer         &buf           = m_decoderThread->decoder()->getFirstBuffer();
-        const char         *data          = buf.currentPointer();
-        size_t              size          = min(buf.remainingSize(), free_size);
+        QByteArray          data          = buf.remainingData();
+        size_t              dataSize      = data.size();
+        size_t              size          = min(dataSize, free_size);
                             consumed_size = SIZE_T_DONT_CARE;
         const SoundMetaData &md           = buf.metaData();
         const SoundFormat   &sf           = buf.soundFormat();
@@ -837,11 +924,13 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
 
 //         logDebug(QString("InternetRadio::noticeReadyForPlaybackData: PLAY: buf_size = %1, min_count = %2, buf_count = %3").arg(size).arg(min_size).arg(n_bufs));
 
-        notifySoundStreamData(id, sf, data, size, consumed_size, md);
+        notifySoundStreamData(id, sf, data.data(), size, consumed_size, md);
 
         if (consumed_size == SIZE_T_DONT_CARE) {
             consumed_size = size;
         }
+//         printf("PLAY: buf_size = %zi, min_count = %i, buf_count = %i, free_size=%zi, avail_size=%zi\n", size, min_size, n_bufs, free_size, dataSize);
+//         printf("     consumed: %zi\n", consumed_size);
         free_size -= consumed_size;
         buf.addProcessedSize(consumed_size);
         if (m_decoderThread && buf.remainingSize() <= 0) {
@@ -852,46 +941,17 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
 }
 
 
-
 bool InternetRadio::checkDecoderMessages()
 {
-    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->warning()) {
-        logWarning(m_decoderThread->decoder()->warningString());
-    }
-    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->debug()) {
-        logDebug(m_decoderThread->decoder()->debugString());
-    }
-    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->error()) {
-        logError(m_decoderThread->decoder()->errorString());
+    bool errorsDetected = !checkLogs(m_decoderThread ? m_decoderThread->decoder() : NULL, i18n("Internet Radio Plugin: "), /*resetLogs = */ true);
+    if (errorsDetected) {
         powerOff();
-        return false;
     }
-    return true;
+    return !errorsDetected;
 }
 
 
 
-bool InternetRadio::event(QEvent *_e)
-{
-    if (SoundStreamDecodingEvent::isSoundStreamDecodingEvent(_e)) {
-        SoundStreamDecodingEvent *e  = static_cast<SoundStreamDecodingEvent*>(_e);
-
-        checkDecoderMessages();
-
-        if (m_decoderThread && m_decoderThread->decoder()) {
-            if (e->type() == SoundStreamDecodingTerminated) {
-                DecoderThread *t = m_decoderThread;
-                m_decoderThread = NULL;
-                t->deleteLater();
-            }
-        }
-        return true;
-    } else {
-        return QObject::event(_e);
-    }
-}
-
-
 void InternetRadio::freeAllBuffers()
 {
     if (m_decoderThread && m_decoderThread->decoder()) {
@@ -968,11 +1028,11 @@ bool InternetRadio::startCaptureWithFormat(SoundStreamID      id,
         powerOn();
     }
 
-    for (int i = 0; i < 20 && isPowerOn() && !(m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->initDone()); ++i) {
+    for (int i = 0; i < 100 && isPowerOn() && !(m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->initDone()); ++i) {
         QEventLoop loop;
         QTimer::singleShot(200, &loop, SLOT(quit()));
         loop.exec();
-    } // wait max 4 secs*/
+    } // wait max 20 secs*/
     if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->initDone()) {
         real_format = m_decoderThread->decoder()->soundFormat();
         return true;
@@ -1000,316 +1060,146 @@ const InternetRadioStation *InternetRadio::findMatchingStation(const StationList
 
 
 
-void InternetRadio::loadPlaylistStopJob()
+void InternetRadio::slotMetaDataUpdate(KIO::MetaData metadata)
 {
-    if (m_playlistJob) {
-        QObject::disconnect(m_playlistJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotPlaylistData(KIO::Job *, const QByteArray &)));
-        QObject::disconnect(m_playlistJob, SIGNAL(result(KJob *)),                        this, SLOT(slotPlaylistLoadDone(KJob *)));
-        m_playlistJob->kill();
-        m_playlistJob = NULL;
+    if (isPowerOn() && metadata.contains("StreamTitle")) {
+        QString title = metadata["StreamTitle"];
+        updateRDSRadioText(title);
+        updateRDSState(true);
     }
 }
 
-void InternetRadio::loadPlaylistStartJob()
-{
-    loadPlaylistStopJob();
-    // if the a new station is selected while the previous hasn't been loaded yet
-    m_playlistData   .clear();
-    m_currentPlaylist.clear();
-    if (getPlaylistClass().length()) {
-        m_playlistJob = KIO::get(m_currentStation.url(), KIO::NoReload, KIO::HideProgressInfo);
-        if (m_playlistJob) {
-            QObject::connect(m_playlistJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotPlaylistData(KIO::Job *, const QByteArray &)));
-            QObject::connect(m_playlistJob, SIGNAL(result(KJob *)),                         this, SLOT(slotPlaylistLoadDone(KJob *)));
-            m_playlistJob->start();
-            if (m_playlistJob->error()) {
-                logError(i18n("Failed to load playlist %1: %2").arg(m_currentStation.url().pathOrUrl()).arg(m_playlistJob->errorString()));
-                powerOff();
-            }
-        } else {
-            logError(i18n("Failed to start playlist download of %1: KIO::get returned NULL pointer").arg(m_currentStation.url().pathOrUrl()));
-            powerOff();
-        }
-    } else {
-        interpretePlaylistData(QByteArray());
-        startDecoderThread();
-    }
-}
-
-
-void InternetRadio::slotPlaylistData(KIO::Job *job, const QByteArray &data)
-{
-    if (job == m_playlistJob) {
-        m_playlistData.append(data);
-    }
-}
-
-
-void InternetRadio::slotPlaylistLoadDone(KJob *job)
-{
-    if (job == m_playlistJob) {
-        if (m_playlistJob->error()) {
-            logError(i18n("Failed to load playlist %1: %2").arg(m_currentStation.url().pathOrUrl()).arg(m_playlistJob->errorString()));
-            powerOff();
-        } else {
-            interpretePlaylistData(m_playlistData);
-            startDecoderThread();
-        }
-        m_playlistJob = NULL;
-    }
-    job->deleteLater();
-}
-
-
-QString InternetRadio::getPlaylistClass()
-{
-    QString plscls = m_currentStation.playlistClass();
-    QString path   = m_currentStation.url().path();
-    if        (plscls == "lsc" || (plscls == "auto" && path.endsWith(".lsc"))) {
-            return "lsc";
-    } else if (plscls == "m3u" || (plscls == "auto" && path.endsWith(".m3u"))) {
-            return "m3u";
-    } else if (plscls == "asx" || (plscls == "auto" && path.endsWith(".asx"))) {
-            return "asx";
-    } else if (plscls == "pls" || (plscls == "auto" && path.endsWith(".pls"))) {
-            return "pls";
-    } else {
-            return "";
-    }
-}
-
-
-void InternetRadio::interpretePlaylistData(const QByteArray &a)
-{
-//     IErrorLogClient::staticLogDebug("InternetRadio::interpretePlaylist");
-    QString plscls = getPlaylistClass();
-    m_currentPlaylist.clear();
-    if        (plscls == "lsc") {
-            interpretePlaylistLSC(a);
-    } else if (plscls == "m3u") {
-            interpretePlaylistM3U(a);
-    } else if (plscls == "asx") {
-            interpretePlaylistASX(a);
-    } else if (plscls == "pls") {
-            interpretePlaylistPLS(a);
-    } else {
-        m_currentPlaylist.append(m_currentStation.url());
-    }
-    if (!m_currentPlaylist.size()) {
-        logError(i18n("%1 does not contain any usable radio stream", m_currentStation.url().pathOrUrl()));
-    }
-
-/*    logDebug("Playlist:");
-    foreach (KUrl url, m_currentPlaylist) {
-        logDebug(url.pathOrUrl());
-    }*/
-}
-
-
-void InternetRadio::interpretePlaylistLSC(const QByteArray &a)
-{
-    interpretePlaylistM3U(a);
-    if (!m_currentPlaylist.size()) {
-        interpretePlaylistASX(a);
-    }
-}
-
-void InternetRadio::interpretePlaylistM3U(const QByteArray &playlistData)
-{
-    QStringList lines = QString(playlistData).split("\n");
-    foreach (QString line, lines) {
-        QString t = line.trimmed();
-        if (t.length() > 5 && !t.startsWith("#")) {
-            m_currentPlaylist.append(t);
-        }
-    }
-}
-
-
-void InternetRadio::interpretePlaylistPLS(const QByteArray &playlistData)
-{
-    KTemporaryFile tmpFile;
-    tmpFile.setAutoRemove(true);
-    if (!tmpFile.open()) {
-        logError(i18n("failed to create temporary file to store playlist data"));
-        return;
-    }
-    if (tmpFile.write(playlistData) != playlistData.size()) {
-        logError(i18n("failed to write temporary file to store playlist data"));
-        return;
-    }
-    tmpFile.close();
-
-    KConfig      cfg(tmpFile.fileName());
-
-    // mapping group names to lower case in order to be case insensitive
-    QStringList            groups = cfg.groupList();
-    QMap<QString, QString> group_lc_map;
-    QString                grp;
-    foreach(grp, groups) {
-        group_lc_map.insert(grp.toLower(), grp);
-    }
-
-    KConfigGroup cfggrp = cfg.group(group_lc_map["playlist"]);
-
-    // mapping entry keys to lower case in order to be case insensitive
-    QStringList keys = cfggrp.keyList();
-    QMap<QString, QString> key_lc_map;
-    QString key;
-    foreach(key, keys) {
-        key_lc_map.insert(key.toLower(), key);
-    }
-
-    unsigned int entries = cfggrp.readEntry(key_lc_map["numberofentries"], 0);
-    if (entries) {
-        for (unsigned int i = 0; i < entries; ++i) {
-            QString url = cfggrp.readEntry(key_lc_map[QString("file%1").arg(i)], QString());
-            if (url.length()) {
-                m_currentPlaylist.append(url);
-            }
-        }
-    }
-}
-
-
-void InternetRadio::interpretePlaylistASX(const QByteArray &rawData)
-{
-
-    KEncodingProber prober;
-    prober.feed(rawData);
-
-    QXmlStreamReader reader(QTextCodec::codecForName(prober.encoding())->toUnicode(rawData));
-
-    bool inEntry = false;
-
-    while (!reader.atEnd() && (reader.error() == QXmlStreamReader::NoError)) {
-        reader.readNext();
-        if (reader.isStartElement()) {
-            QStringRef name = reader.name();
-            if (name.toString().toLower() == "entry") {
-                inEntry = true;
-            }
-            else if (name.toString().toLower() == "ref" && inEntry) {
-                QXmlStreamAttributes attrs = reader.attributes();
-                QXmlStreamAttribute  attr;
-                foreach(attr, attrs) {
-                    if(attr.name().toString().toLower() == "href") {
-                        m_currentPlaylist.append(attr.value().toString());
-                    }
-                }
-            }
-        }
-        else if (reader.isEndElement()) {
-            QStringRef name = reader.name();
-            if (name == "entry") {
-                inEntry = false;
-            }
-        }
-    }
-
-    if (reader.error() != QXmlStreamReader::NoError) {
-        logError(i18n("error while reading asx file: ", reader.error()));
-    }
-}
-
-
-
-
 
 
 #ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
 
-void InternetRadio::startStreamDownload()
-{
-    m_randStreamIdxOffset      = rint((m_currentPlaylist.size() - 1) * (float)rand() / (float)RAND_MAX);
-    m_currentStreamIdx         = 0;
-    m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
 
-    tryNextStream();
+void InternetRadio::slotStreamError(KUrl /*url*/)
+{
+    m_playlistHandler.selectNextStream(true, true);
 }
 
 
-void InternetRadio::tryNextStream()
+void InternetRadio::slotStreamFinished(KUrl /*url*/)
 {
-    do {
-        if (--m_currentStreamRetriesLeft < 0) {
-            ++m_currentStreamIdx;
-            m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
-        }
-        if (isPowerOn() && m_currentStreamIdx < m_currentPlaylist.size()) {
-            m_currentStreamUrl = m_currentPlaylist[(m_currentStreamIdx + m_randStreamIdxOffset) % m_currentPlaylist.size()];
-
-            // start download job
-            stopStreamDownload();
-            logDebug(i18n("opening stream %1", m_currentStreamUrl.pathOrUrl()));
-
-            m_streamJob = KIO::get(m_currentStreamUrl, KIO::NoReload, KIO::HideProgressInfo);
-            if (m_streamJob) {
-                QObject::connect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
-                QObject::connect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
-                m_streamJob->start();
-                if (m_streamJob->error()) {
-                    logError(i18n("Failed to start stream download of %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
-                    stopStreamDownload();
-                }
-            } else {
-                logError(i18n("Failed to start stream download of %1: KIO::get returned NULL pointer").arg(m_currentStreamUrl.pathOrUrl()));
-                stopStreamDownload();
-            }
-        } else {
-            logError(i18n("Failed to start any stream of %1").arg(m_currentStation.longName()));
-            powerOff();
-        }
-    } while (isPowerOn() && !m_streamJob);
+    powerOff();
 }
 
 
-void InternetRadio::stopStreamDownload()
+void InternetRadio::slotStreamStarted(KUrl /*url*/)
 {
-    if (m_streamJob) {
-        QObject::disconnect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
-        QObject::disconnect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
-        m_streamJob->kill();
-        m_streamJob = NULL;
-    }
+    // currently ignored
 }
 
 
-void InternetRadio::slotStreamData(KIO::Job *job, const QByteArray &data)
+void InternetRadio::slotStreamConnectionEstablished(KUrl /*url*/, KIO::MetaData /*metaData*/)
 {
-    if (m_streamJob == job) {
-//         logDebug(QString("stream data: %1 bytes").arg(data.size()));
-        if (m_decoderThread && m_decoderThread->decoder()) {
-            bool isfull = false;
-            m_decoderThread->decoder()->writeInputBuffer(data, isfull, m_currentStreamUrl);
-            if (isfull) {
-                m_streamJob->suspend();
-            }
-        }
-    }
+    startDecoderThread();
 }
 
 
-void InternetRadio::slotStreamContinue()
+void    InternetRadio::slotInputStreamUrlChanged(KUrl /*url*/)
 {
-    if (m_streamJob) {
-        m_streamJob->resume();
-    }
-}
-
-
-void InternetRadio::slotStreamDone(KJob *job)
-{
-    if (m_streamJob == job) {
-        if (m_streamJob->error()) {
-            logError(i18n("Failed to load stream data for %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
-            tryNextStream();
-        }
-        stopStreamDownload();
-    }
-    job->deleteLater();
+    // currently ignored
 }
 
+//
+// void InternetRadio::startStreamDownload()
+// {
+//     m_randStreamIdxOffset      = rint((m_currentPlaylist.size() - 1) * (float)rand() / (float)RAND_MAX);
+//     m_currentStreamIdx         = 0;
+//     m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
+//
+//     tryNextStream();
+// }
+//
+//
+// void InternetRadio::tryNextStream()
+// {
+//     do {
+//         if (--m_currentStreamRetriesLeft < 0) {
+//             ++m_currentStreamIdx;
+//             m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
+//         }
+//         if (isPowerOn() && m_currentStreamIdx < m_currentPlaylist.size()) {
+//             m_currentStreamUrl = m_currentPlaylist[(m_currentStreamIdx + m_randStreamIdxOffset) % m_currentPlaylist.size()];
+//
+//             // start download job
+//             stopStreamDownload();
+//             logDebug(i18n("opening stream %1", m_currentStreamUrl.pathOrUrl()));
+//
+//             m_streamJob = KIO::get(m_currentStreamUrl, KIO::NoReload, KIO::HideProgressInfo);
+//             if (m_streamJob) {
+//                 QObject::connect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
+//                 QObject::connect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
+//                 m_streamJob->start();
+//                 if (m_streamJob->error()) {
+//                     logError(i18n("Failed to start stream download of %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
+//                     stopStreamDownload();
+//                 }
+//             } else {
+//                 logError(i18n("Failed to start stream download of %1: KIO::get returned NULL pointer").arg(m_currentStreamUrl.pathOrUrl()));
+//                 stopStreamDownload();
+//             }
+//         } else {
+//             logError(i18n("Failed to start any stream of %1").arg(m_currentStation.longName()));
+//             powerOff();
+//         }
+//     } while (isPowerOn() && !m_streamJob);
+// }
+//
+//
+// void InternetRadio::stopStreamDownload()
+// {
+//     if (m_streamJob) {
+//         QObject::disconnect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
+//         QObject::disconnect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
+//         m_streamJob->kill();
+//         m_streamJob = NULL;
+//     }
+// }
+//
+//
+// void InternetRadio::slotStreamData(KIO::Job *job, const QByteArray &data)
+// {
+//     if (m_streamJob == job) {
+// //         logDebug(QString("stream data: %1 bytes").arg(data.size()));
+//         if (m_decoderThread && m_decoderThread->decoder()) {
+//             bool isfull = false;
+//             m_decoderThread->decoder()->writeInputBuffer(data, isfull, m_currentStreamUrl);
+//             if (isfull) {
+//                 m_streamJob->suspend();
+//             }
+//         }
+//     }
+// }
+//
+//
+// void InternetRadio::slotStreamContinue()
+// {
+//     if (m_streamJob) {
+//         m_streamJob->resume();
+//     }
+// }
+//
+//
+// void InternetRadio::slotStreamDone(KJob *job)
+// {
+//     if (m_streamJob == job) {
+//         bool err = false;
+//         if (m_streamJob->error()) {
+//             logError(i18n("Failed to load stream data for %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
+//             err = true;
+//         }
+//         stopStreamDownload();
+//         if (err) {
+//             m_currentStreamRetriesLeft = 0;
+//             tryNextStream();
+//         }
+//     }
+//     job->deleteLater();
+// }
+//
 #endif
 
 
