@@ -48,6 +48,8 @@
 #include <kurlrequester.h>
 #include <klocale.h>
 #include <kmenu.h>
+#include <kdemacros.h>
+#include <kdeversion.h>
 
 RadioConfiguration::RadioConfiguration (QWidget *parent, const IErrorLogClient &logger)
     : QWidget(parent),
@@ -59,7 +61,13 @@ RadioConfiguration::RadioConfiguration (QWidget *parent, const IErrorLogClient &
 {
     setupUi(this);
 
-    editPresetFile->setPath(KGlobal::dirs()->saveLocation("data", "kradio4"));
+    QString defaultPresetDir = KGlobal::dirs()->saveLocation("data", "kradio4");
+
+    #if KDE_IS_VERSION(4, 2, 80)
+        editPresetFile->setStartDir(defaultPresetDir);
+    #else
+        editPresetFile->setPath    (defaultPresetDir);
+    #endif
 
     // icon settings does not work any more in .ui files in KDE4, don't know why/how
     buttonNewStation      ->setIcon(KIcon("document-new"));
@@ -106,8 +114,6 @@ RadioConfiguration::RadioConfiguration (QWidget *parent, const IErrorLogClient &
                      this, SLOT(slotStationDown()));
     QObject::connect(listStations, SIGNAL(sigStationActivated(int)),
                      this, SLOT(slotActivateStation( int )));
-    QObject::connect(buttonLoadPresets, SIGNAL(clicked()),
-                     this, SLOT(slotLoadPresets()));
     QObject::connect(buttonStorePresets, SIGNAL(clicked()),
                      this, SLOT(slotStorePresets()));
     QObject::connect(buttonLastChangeNow, SIGNAL(clicked()),
@@ -126,9 +132,6 @@ RadioConfiguration::RadioConfiguration (QWidget *parent, const IErrorLogClient &
     QObject::connect(mailLabel, SIGNAL(leftClickedUrl(const QString &)),
                      this, SLOT(slotSendPresetsByMail(const QString &)));
 
-    QObject::connect(buttonSearchStations, SIGNAL(clicked()),
-                     this, SLOT(slotSearchStations0()));
-
     m_devicePopup = new QMenu(buttonSearchStations);
     buttonSearchStations->setMenu(m_devicePopup);
     QObject::connect(m_devicePopup, SIGNAL(triggered(QAction*)),
@@ -208,10 +211,16 @@ bool RadioConfiguration::noticeStationsChanged(const StationList &sl)
 {
     m_ignoreChanges = true;
 
+    QString old_id = m_stations.at(listStations->currentStationIndex()).stationID();
+
     m_stations = sl;
 
     listStations->setStations(sl);
 
+    listStations->setCurrentStation(m_stations.idxWithID(old_id));
+    listStations->ensureItemVisible(listStations->currentItem());
+
+
     StationListMetaData &info = m_stations.metaData();
 
     editMaintainer->setText(info.maintainer);
@@ -270,14 +279,6 @@ void RadioConfiguration::slotStationSelectionChanged(int idx)
     int stModeIdx         = comboStereoMode->findData(s ? (int)s->stereoMode() : (int)STATION_STEREO_DONTCARE);
     comboStereoMode       ->setCurrentIndex(stModeIdx >= 0 ? stModeIdx : comboStereoMode->findData((int)STATION_STEREO_DONTCARE));
 
-    QPixmap pixmap(s ? s->iconName() : QString());
-    if (!pixmap.isNull()) {
-        pixmapStation->setPixmap(pixmap);
-    } else {
-        pixmapStation->setPixmap(QPixmap());
-    }
-
-
     stackStationEdit->setDisabled(!s);
     if (s) {
         QMap<QString, RadioStationConfig*>::const_iterator it_c = m_stationEditors.find(s->getClassName());
@@ -326,14 +327,27 @@ void RadioConfiguration::slotNewStation(QAction *a)
 
 void RadioConfiguration::createNewStation(const RadioStation *rs_template)
 {
-    RadioStation       *newst = rs_template->copyNewID();
-    int n = m_stations.count();
+    RadioStation   *newst      = rs_template->copyNewID();
+    int             currentIdx = listStations->currentStationIndex();
+    int             n          = m_stations.count();
+    if (currentIdx < 0) {
+        currentIdx = n - 1;
+    }
+
     m_stations.addStation(*newst);
     if (m_stations.count() > n) {
+        m_stations.moveStation(n, currentIdx + 1);
+
+        m_ignoreChanges = true;
         listStations->appendStation(*newst);
-        listStations->setCurrentStation (listStations->count()-1);
-        slotStationSelectionChanged(listStations->count()-1);
-        listStations->ensureItemVisible(listStations->selectedItem());
+        for (int i = currentIdx + 1; i <= n; ++i) {
+            listStations->setStation(i, m_stations.at(i));
+        }
+        listStations->setCurrentStation(currentIdx + 1);
+        listStations->ensureItemVisible(listStations->currentItem());
+        m_ignoreChanges = false;
+        slotStationSelectionChanged(listStations->currentStationIndex());
+
     }
     delete newst;
 }
@@ -437,7 +451,6 @@ void RadioConfiguration::slotPixmapChanged( const QString &s )
         RadioStation &st = m_stations.at(idx);
         st.setIconName(s);
         m_ignoreChanges = true;
-        pixmapStation->setPixmap(QPixmap(s));
         bool o = listStations->signalsBlocked();
         listStations->blockSignals(true);
         listStations->setStation(idx, st);
@@ -489,6 +502,7 @@ void RadioConfiguration::slotStationUp()
         listStations->setStation(idx-1, sl.at(idx-1));
         listStations->setStation(idx,   sl.at(idx));
         listStations->setCurrentStation(idx-1);
+        listStations->ensureItemVisible(listStations->currentItem());
 //         listStations->blockSignals(o);
         m_ignoreChanges = false;
     }
@@ -510,6 +524,7 @@ void RadioConfiguration::slotStationDown()
         listStations->setStation(idx,   sl.at(idx));
         listStations->setStation(idx+1, sl.at(idx+1));
         listStations->setCurrentStation(idx+1);
+        listStations->ensureItemVisible(listStations->currentItem());
 //         listStations->blockSignals(o);
         m_ignoreChanges = false;
     }
@@ -633,7 +648,7 @@ void RadioConfiguration::slotSendPresetsByMail( const QString &url )
 
 void RadioConfiguration::slotSearchStations(QAction *a)
 {
-    int idev = a->data().toInt();
+    int idev = a ? a->data().toInt() : -1;
     if (idev >= 0 && idev < m_devices.count()) {
         IRadioDevice *dev = m_devices.at(idev);
 
